// $Id: ConnectionAttributes.cpp,v 1.14 2014/02/10 20:50:48 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include <algorithm>

#include "Command.h"
#include "Debug.h"
#include "ConnectionAttributes.h"
#include "XmlDef.h"
#include "XmlOptions.h"



//_________________________________________________________
ConnectionAttributes::ConnectionAttributes( void ):
  Counter( "ConnectionAttributes" )
{ Debug::Throw( "ConnectionAttributes::ConnectionAttributes.\n" ); }

//_________________________________________________________
ConnectionAttributes::ConnectionAttributes( const QDomElement& element ):
  Counter( "ConnectionAttributes" )
{
  Debug::Throw( "ConnectionAttributes::ConnectionAttributes.\n" );

  // parse attributes
  QDomNamedNodeMap attributes( element.attributes() );
  for( unsigned int i=0; i<attributes.length(); i++ )
  {

    QDomAttr attribute( attributes.item( i ).toAttr() );
    if( attribute.isNull() ) continue;
    QString name( attribute.name() );
    QString value( attribute.value() );

    if( name == Xml::Name ) setName( value );
    else if( name == Xml::Host ) setHost( value );
    else if( name == Xml::User ) setUser( value );
    else if( name == Xml::Options ) setOptions( value );
    else if( name == Xml::AutoConnect ) setAutoConnect( value.toInt() );
    else Debug::Throw(0) << "ConnectionAttributes::ConnectionAttributes - unrecognized name " << name << endl;

  }

  // loop over children
  // parse children
  for(QDomNode child_node = element.firstChild(); !child_node.isNull(); child_node = child_node.nextSibling() )
  {
    QDomElement child_element = child_node.toElement();
    if( child_element.isNull() ) continue;
    if( child_element.tagName() == Xml::Tunnel ) tunnelAttributes_ << TunnelAttributes( child_element );
    else Debug::Throw(0) << "ConnectionAttributes::ConnectionAttributes - unrecognized chile " << child_element.tagName() << endl;
  }

}

//_________________________________________________________
QDomElement ConnectionAttributes::domElement( QDomDocument& document ) const
{
  Debug::Throw( "ConnectionAttributes::domElement.\n" );
  QDomElement out( document.createElement( Xml::Connection ) );

  // attributes
  out.setAttribute( Xml::Name, name() );
  out.setAttribute( Xml::Host, host() );
  out.setAttribute( Xml::User, user() );
  out.setAttribute( Xml::Options, options() );
  out.setAttribute( Xml::AutoConnect, QString::number( autoConnect() ) );

  // write tunnels
  for( TunnelAttributes::List::const_iterator iter = tunnelAttributes_.begin(); iter != tunnelAttributes_.end(); ++iter )
  { out.appendChild( iter->domElement( document ) ); }

  return out;

}

//_________________________________________________________
QStringList ConnectionAttributes::errors( void ) const
{
  QStringList out;
  if( host().isEmpty() ) out << QObject::tr( "  Invalid hostname;" );
  if( tunnelAttributes().empty() ) out << QObject::tr( " No tunnels set;" );
  else if( std::find_if( tunnelAttributes().begin(), tunnelAttributes().end(), TunnelAttributes::ValidFTor() ) == tunnelAttributes().end()  )
  { out << QObject::tr( "  No valid tunnels set;" ); }

  return out;

}

//_________________________________________________________
bool ConnectionAttributes::isValid( void ) const
{

  return !(
    host().isEmpty() ||
    std::find_if( tunnelAttributes().begin(), tunnelAttributes().end(), TunnelAttributes::ValidFTor() ) == tunnelAttributes().end() );

}

//_________________________________________________________
QString ConnectionAttributes::command( void ) const
{
  Debug::Throw( "ConnectionAttributes::command.\n" );

  if( !isValid() ) return "";

  QString out( XmlOptions::get().raw( "SSH_COMMAND" ) );
  QTextStream what( &out );
  if( !options().isEmpty() ) { what << " " << options(); }

  // add tunnels
  for( TunnelAttributes::List::const_iterator iter = tunnelAttributes().begin(); iter != tunnelAttributes().end(); ++iter )
  {
    if( !iter->isValid() ) continue;
    what << " -L " << iter->localPort() << ":" << iter->host() << ":" << iter->remotePort();
  }

  // login
  what << " ";
  if( !user().isEmpty() ) what << user() << "@";
  what << host();

  return out;

}
