// $Id: ConnectionListDialog.cpp,v 1.15 2014/02/03 10:08:26 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionListDialog.h"
#include "ConnectionListDialog.moc"

#include "AnimatedTreeView.h"
#include "ConnectionDialog.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "InformationDialog.h"
#include "QuestionDialog.h"
#include "SimpleListView.h"

#include <QLabel>
#include <QLayout>

//___________________________________________________________
ConnectionListDialog::ConnectionListDialog( QWidget* parent ):
    CustomDialog(parent, OkButton|CancelButton )
{

    Debug::Throw( "ConnectionListDialog::ConnectionListDialog.\n" );
    setOptionName( "CONNECTION_LIST_DIALOG" );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    mainLayout().addLayout( hLayout );

    hLayout->addWidget( list_ = new SimpleListView( this ), 0 );
    _list().setModel( &model_ );
    _list().setSortingEnabled( false );

    QVBoxLayout* vLayout = new QVBoxLayout();
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    hLayout->addLayout( vLayout, 1 );

    // buttons
    vLayout->addWidget( addButton_ = new QPushButton( IconEngine::get( IconNames::Add ), tr( "Add" ), this ) );
    vLayout->addWidget( removeButton_ = new QPushButton( IconEngine::get( IconNames::Remove ), tr( "Remove" ), this ) );
    vLayout->addWidget( editButton_ = new QPushButton( IconEngine::get( IconNames::Edit ), tr( "Edit" ), this ) );
    vLayout->addWidget( moveUpButton_ = new QPushButton( IconEngine::get( IconNames::Up ), tr( "Move Up" ), this ) );
    vLayout->addWidget( moveDownButton_ = new QPushButton( IconEngine::get( IconNames::Down ), tr( "Move Down" ), this ) );
    vLayout->addStretch( 1 );

    addButton_->setToolTip( tr( "Add a new connection" ) );
    removeButton_->setToolTip( tr( "Remove selected connection" ) );
    editButton_->setToolTip( tr( "Edit selected connection properties" ) );
    moveUpButton_->setToolTip( tr( "Move selected connection up in the list" ) );
    moveDownButton_->setToolTip( tr( "Move selected connection down in the list" ) );


    connect( addButton_, SIGNAL(pressed()), SLOT(_add()) );
    connect( removeButton_, SIGNAL(pressed()), SLOT(_remove()) );
    connect( editButton_, SIGNAL(pressed()), SLOT(_edit()) );
    connect( moveUpButton_, SIGNAL(pressed()), SLOT(_up()) );
    connect( moveDownButton_, SIGNAL(pressed()), SLOT(_down()) );
    connect( &_list(), SIGNAL(activated(QModelIndex)), SLOT(_edit()) );

    connect( _list().selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(_updateButtons()) );
    connect( _list().selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), SLOT(_updateButtons()) );

    okButton().setText( tr( "Save Connections" ) );
    okButton().setIcon( IconEngine::get( IconNames::Save ) );

    _updateButtons();

}

//________________________________________
void ConnectionListDialog::setConnectionAttributes( const ConnectionAttributes::List& attributesList )
{
    Debug::Throw( "ConnectionListDialog::setConnectionAttributes.\n" );
    _model().set( attributesList );
    _list().resizeColumns();
}

//________________________________________
ConnectionAttributes::List ConnectionListDialog::connectionAttributes( void ) const
{
    Debug::Throw( "ConnectionListDialog::connectionAttributes.\n" );
    return _model().get();
}

//________________________________________
void ConnectionListDialog::_updateButtons( void )
{
    Debug::Throw( "ConnectionListDialog::_updateButtons.\n" );

    bool hasSelection( !_list().selectionModel()->selectedRows().empty() );
    editButton_->setEnabled( hasSelection );
    removeButton_->setEnabled( hasSelection );

    moveUpButton_->setEnabled( hasSelection && !_list().selectionModel()->isSelected( model_.index( 0, 0 ) ) );
    moveDownButton_->setEnabled( hasSelection && !_list().selectionModel()->isSelected( model_.index( model_.rowCount()-1, 0 ) ) );

}

//________________________________________
void ConnectionListDialog::_add( void )
{

    Debug::Throw( "ConnectionListDialog::_add.\n" );
    ConnectionAttributes attributes;
    while( !attributes.isValid() )
    {

        // create dialog
        ConnectionDialog dialog( this );
        dialog.setConnectionAttributes( attributes );
        if( !dialog.centerOnParent().exec() ) return;

        // get attributes and check validity
        attributes = dialog.connectionAttributes();
        if( !attributes.isValid() )
        {
            const QString buffer = QString( tr( "Connection settings are invalid:\n%1\nModify ?" ) )
                .arg( attributes.errors().join( "\n" ) );

            if( !QuestionDialog( this, buffer ).centerOnWidget( window() ).exec() )
            { break; }
        }

    }

    // check name unicity
    if( _checkUnicity( attributes ) ) _model().add( attributes );

}

//________________________________________
void ConnectionListDialog::_remove( void )
{

    Debug::Throw( "ConnectionListDialog::_remove.\n" );

    // retrieve selected items; retrieve only recordable options
    ConnectionAttributesModel::List selection( _model().get( _list().selectionModel()->selectedRows() ) );
    ConnectionAttributesModel::List removed;
    for( ConnectionAttributesModel::List::const_iterator iter = selection.begin(); iter != selection.end(); ++iter )
    { removed << *iter; }

    // remove
    _model().remove( removed );
    _list().resizeColumns();

}

//________________________________________
void ConnectionListDialog::_edit( void )
{

    Debug::Throw( "ConnectionListDialog::_edit.\n" );
    QModelIndex current( _list().selectionModel()->currentIndex() );
    if( !current.isValid() )
    {
        InformationDialog( this, tr( "No item selected. <Edit> canceled." ) ).exec();
        return;
    }

    // retrieve DocumentClass matching name
    ConnectionAttributes attributes( _model().get( current ) );
    ConnectionDialog dialog( this );
    dialog.setConnectionAttributes( attributes );
    if( !dialog.centerOnParent().exec() ) return;

    // remove old attributes
    _model().remove( attributes );

    // add new attributes
    attributes = dialog.connectionAttributes();

    // check validity
    // check validity
    if( !attributes.isValid() )
    {
        const QString buffer = QString( tr( "Tunnel settings are invalid:\n%1" ) ).arg( attributes.errors().join( "\n" ) );
        InformationDialog( this, buffer ).centerOnWidget( window() ).exec();
    }

    // add to model_
    if( _checkUnicity( attributes ) ) _model().add( attributes );

    // make sure item is selected
    QModelIndex index( _model().index( attributes ) );
    if( index != _list().selectionModel()->currentIndex() )
    {
        _list().selectionModel()->select( index,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        _list().selectionModel()->setCurrentIndex( index,  QItemSelectionModel::Current|QItemSelectionModel::Rows );
    }

    _list().resizeColumns();

}

//_________________________________________________________________________
void ConnectionListDialog::_up( void )
{

    Debug::Throw( "ConnectionListDialog::_up.\n" );
    ConnectionAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
    if( selection.empty() ) return;

    // retrieve selected indexes in list and store in model
    QModelIndexList selectedIndexes( _list().selectionModel()->selectedRows() );
    ConnectionAttributes::List selectedAttributes( model_.get( selectedIndexes ) );

    ConnectionAttributes::List currentAttributes( connectionAttributes() );
    ConnectionAttributes::List newAttributes;

    for( ConnectionAttributes::List::const_iterator iter = currentAttributes.begin(); iter != currentAttributes.end(); ++iter )
    {

        // check if new list is not empty, current index is selected and last index is not.
        // if yes, move.
        if(
            !( newAttributes.empty() ||
            selectedIndexes.indexOf( model_.index( *iter ) ) == -1 ||
            selectedIndexes.indexOf( model_.index( newAttributes.back() ) ) != -1
            ) )
        {
            ConnectionAttributes last( newAttributes.back() );
            newAttributes.removeLast();
            newAttributes << *iter;
            newAttributes << last;
        } else newAttributes << *iter;

    }

    model_.set( newAttributes );

    // restore selection
    _list().selectionModel()->select( model_.index( selectedAttributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( ConnectionAttributes::List::const_iterator iter = selectedAttributes.begin(); iter != selectedAttributes.end(); ++iter )
    { _list().selectionModel()->select( model_.index( *iter ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

}

//_________________________________________________________________________
void ConnectionListDialog::_down( void )
{

    Debug::Throw( "ConnectionListDialog::_down.\n" );
    ConnectionAttributes::List selection( model_.get( _list().selectionModel()->selectedRows() ) );
    if( selection.empty() ) return;

    // retrieve selected indexes in list and store in model
    QModelIndexList selectedIndexes( _list().selectionModel()->selectedIndexes() );
    ConnectionAttributes::List selectedAttributes( model_.get( selectedIndexes ) );

    ConnectionAttributes::List currentAttributes( ConnectionListDialog::connectionAttributes() );
    ConnectionAttributes::List newAttributes;
    ConnectionAttributes::ListIterator iter( currentAttributes );
    iter.toBack();
    while( iter.hasPrevious() )
    {
        const ConnectionAttributes current( iter.previous() );

        // check if new list is not empty, current index is selected and last index is not.
        // if yes, move.
        if(
            !( newAttributes.empty() ||
            selectedIndexes.indexOf( model_.index( current ) ) == -1 ||
            selectedIndexes.indexOf( model_.index( newAttributes.front() ) ) != -1
            ) )
        {

            ConnectionAttributes first( newAttributes.front() );
            newAttributes.removeFirst();
            newAttributes.push_front( current );
            newAttributes.push_front( first );

        } else newAttributes.push_front( current );
    }

    model_.set( newAttributes );

    // restore selection
    _list().selectionModel()->select( model_.index( selectedAttributes.front() ),  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
    for( ConnectionAttributes::List::const_iterator iter = selectedAttributes.begin(); iter != selectedAttributes.end(); ++iter )
    { _list().selectionModel()->select( model_.index( *iter ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    return;
}

//_________________________________________________________________________
bool ConnectionListDialog::_checkUnicity( ConnectionAttributes& attributes )
{

    ConnectionAttributes::List attributes_list( _model().get() );
    while( std::find_if( attributes_list.begin(), attributes_list.end(), ConnectionAttributes::SameNameFTor( attributes.name() ) ) != attributes_list.end() )
    {
        const QString buffer = QString( tr( "A connection named %1 already exists.\n Rename ?" ) ).arg( attributes.name() );
        if( !QuestionDialog( this, buffer ).centerOnWidget( window() ).exec() ) return false;

        ConnectionDialog dialog( this );
        dialog.setConnectionAttributes( attributes );
        if( dialog.centerOnWidget( window() ).exec() )
        { attributes = dialog.connectionAttributes(); }

    }

    return true;

}
