// $Id: MainMenu.cpp,v 1.33 2014/03/16 11:18:03 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainMenu.h"
#include "MainMenu.moc"

#include "ConnectionDialog.h"
#include "ConnectionParser.h"
#include "ConnectionObject.h"
#include "ConnectionListDialog.h"
#include "Debug.h"
#include "IconEngine.h"
#include "IconNames.h"
#include "IconSize.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QStyleOption>
#include <algorithm>

class Route
{
    public:

    //! device
    QString device_;

    //! network
    QString network_;

    //! gateway
    QString gateway_;

    //! metric
    int metric_;

    //! validity
    bool valid_;

    // streamer
    friend QTextStream& operator >> ( QTextStream& in, Route& route )
    {
        int dummy;
        in >> route.device_ >> route.network_ >> route.gateway_ >> dummy >> dummy >> dummy >> route.metric_;
        route.valid_ = (in.status() == QTextStream::Ok );
        return in;
    }
};

//____________________________________________________________
MainMenu::MainMenu( QWidget* parent ):
    QMenu( parent ),
    Counter( "MainMenu" ),
    routeFileName_( "/proc/net/route" )
{

    Debug::Throw( "Network::MainMenuMainMenu.\n" );

    _installActions();

    connect( qApp, SIGNAL(aboutToQuit()), SLOT(closeConnections()) );

    setSeparatorsCollapsible( false );
    QAction* separator = addSeparator();
    separator->setText( tr( "Connections" ) );
    separator->setIcon( windowIcon() );

    // icon size
    // calculate minimum size, needed to properly account for header
    QStyleOption opt;
    opt.init( this );
    const int iconSize = style()->pixelMetric(QStyle::PM_SmallIconSize, &opt, this);

    QFont font( MainMenu::font() );
    font.setWeight( QFont::Bold );
    int width = iconSize + QFontMetrics( font ).width( separator->text() ) + 24;
    setMinimumSize( QSize( width, 0 ) );

    addAction( reconnectAction_ );
    addSeparator();

    // check devices
    _checkDevice();

    // configuration
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//____________________________________________________________
MainMenu::~MainMenu( void )
{ Debug::Throw( "Network::MainMenu~MainMenu.\n" ); }

//____________________________________________________________
void MainMenu::readConnections( void )
{

    Debug::Throw( "Network::MainMenureadConnections.\n" );
    ConnectionParser parser( QString( XmlOptions::get().raw( "RC_FILE" ) ) );
    ConnectionAttributes::List attributes = parser.read();
    if( parser.hasError() )
    { emit messageAvailable( parser.error() ); }

    for( ConnectionAttributes::List::const_iterator iter = attributes.begin(); iter != attributes.end(); ++iter )
    { _addConnection( *iter ); }

    if( attributes.empty() )
    { emit messageAvailable( tr( "No connections are presently registered.\n Righ-click to configure connections." ) ); }

}

//____________________________________________________________
unsigned int MainMenu::connectionsCount( void ) const
{ return findChildren<ConnectionObject*>().size(); }

//____________________________________________________________
void MainMenu::autoConnect( void )
{

    Debug::Throw( "Network::MainMenuautoConnect.\n" );

    // get attributes
    foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
    {
        if( object->connectionAttributes().autoConnect() && object->connectionAttributes().isValid() )
        { object->connectAction().setChecked( true ); }
    }


}

//____________________________________________________________
void MainMenu::reconnect( void )
{

    Debug::Throw( "Network::MainMenureconnect.\n" );
    foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
    {
        if( object->connectAction().isChecked() )
        {
            object->connectAction().setChecked( false );
            object->connectAction().setChecked( true );
        }
    }

}

//____________________________________________________________
void MainMenu::closeConnections( void )
{

    Debug::Throw( "Network::MainMenucloseConnections.\n" );
    foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
    {  if( object->connectAction().isChecked() ) object->connectAction().setChecked( false ); }

}

//____________________________________________________________
void MainMenu::newConnection( void )
{
    Debug::Throw( "Network::MainMenunewConnection.\n" );
    ConnectionDialog dialog( this );
    dialog.setWindowTitle( tr( "New Connection - Qssh" ) );
    if( !dialog.centerOnParent().exec() ) return;

    _addConnection( dialog.connectionAttributes() );
    _connectionsChanged();
}

//____________________________________________________________
void MainMenu::showEvent( QShowEvent* )
{

    Debug::Throw( "Network::MainMenushowEvent.\n" );
    bool isConnected( false );
    foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
    {
        if( object->connectAction().isChecked() )
        {
            isConnected = true;
            break;
        }
    }

    reconnectAction_->setEnabled( isConnected );

}

//______________________________________________________________________
void MainMenu::timerEvent( QTimerEvent* event )
{

    if( event->timerId() == deviceTimer_.timerId() )  _checkDevice();
    else return QObject::timerEvent( event );

}

//________________________________________________
MainMenu::DeviceSet MainMenu::_connectedDevices( void ) const
{
    Debug::Throw( "Network::MainMenu_connectedDevices.\n" );

    DeviceSet out;

    // parse routes
    QFile in( routeFileName_ );
    if( !in.open( QIODevice::ReadOnly ) ) return out;

    QStringList lines( QString( in.readAll() ).split( "\n" ) );
    for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
    {

        QTextStream stream( &*iter );
        Route route;
        stream >> route;
        if( !route.valid_ ) continue;
        Debug::Throw() << "Network::MainMenu_connectedDevices - "
            << route.device_ << " "
            << route.network_ << " "
            << route.gateway_ << " "
            << route.metric_
            << endl;

        if( route.device_ == "lo" ) continue;

        // should put more checks
        out.insert( route.device_ );

    }

    return out;

}

//________________________________________________
void MainMenu::_checkDevice( void )
{

    Debug::Throw( "Network::MainMenu::_checkDevice.\n" );

    // get connected devices
    DeviceSet devices( _connectedDevices() );

    // if no connected devices is found, clear current
    if( devices.isEmpty() )
    {
        device_ = QString();
        return;
    }

    // check if current device is in list found
    if( !( device_.isEmpty() || devices.find( device_ ) == devices.end() ) )
    { return; }

    // new device is found
    device_ = *devices.begin();

    // check for automatic reconnection
    const bool automatic( XmlOptions::get().get<bool>( "NETWORK_AUTO_SELECT" ) );
    if( !automatic ) return;

    // reconnect existing connections
    foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
    {

        if( object->connectAction().isChecked() )
        {

            object->connectAction().setChecked( false );
            object->connectAction().setChecked( true );

        } else if( object->connectionAttributes().autoConnect() && object->connectionAttributes().isValid() ) {

            object->connectAction().setChecked( true );

        }

    }

}

//__________________________________________________________
void MainMenu::_updateConfiguration( void )
{
    Debug::Throw( "Network::MainMenu::_updateConfiguration.\n" );

    // stop device timer
    if( deviceTimer_.isActive() ) deviceTimer_.stop();

    // device selection
    const bool automatic( XmlOptions::get().get<bool>( "NETWORK_AUTO_SELECT" ) );
    if( automatic )
    { deviceTimer_.start( 1000*XmlOptions::get().get<int>( "NETWORK_AUTO_SELECT_RATE" ), this ); }

}

//____________________________________________________________
void MainMenu::_connectionsChanged( void )
{

    Debug::Throw( "Network::MainMenu_connectionsChanged.\n" );
    ConnectionAttributes::List attributes;

    // should respect ordering here
    QList<QAction*> actions( this->actions() );
    foreach( QAction* action, actions )
    {

        foreach( ConnectionObject* object, findChildren<ConnectionObject*>() )
        {
            if( &object->connectAction() == action )
            {
                attributes << object->connectionAttributes();
                break;
            }
        }

    }

    ConnectionParser parser( QString( XmlOptions::get().raw( "RC_FILE" ) ) );
    parser.write( attributes );
    if( parser.hasError() )
    { emit messageAvailable( parser.error() ); }

    emit connectionStatusChanged();

}

//____________________________________________________________
void MainMenu::_configureConnections( void )
{
    Debug::Throw( "Network::MainMenu_configureConnections.\n" );

    // get attributes
    ConnectionAttributes::List attributesList;
    QList<ConnectionObject*> objects( findChildren<ConnectionObject*>() );
    foreach( ConnectionObject* object, objects )
    { attributesList << object->connectionAttributes(); }

    // create dialog
    ConnectionListDialog dialog( this );
    dialog.setConnectionAttributes( attributesList );
    if( !dialog.centerOnWidget( window() ).exec() ) return;

    attributesList = dialog.connectionAttributes();

    // delete all parent objects
    // this should be made smarter, in order not to delete objects for which identical attributes are found
    foreach( ConnectionObject* object, objects )
    {

        ConnectionAttributes::List::iterator attributesIter(
            std::find_if(
            attributesList.begin(), attributesList.end(),
            SameConnectionFTor( object->connectionAttributes() ) ) );

        if( attributesIter == attributesList.end() )
        {

            // disconnect and delete connection
            object->connectAction().setChecked( false );
            object->deleteLater();

        } else {

            object->setConnectionAttributes( *attributesIter );
            attributesList.erase( attributesIter );
        }

    }

    foreach( const ConnectionAttributes& attributes, attributesList )
    { _addConnection( attributes ); }

    // first remove all actions
    // objects = Base::KeySet<ConnectionObject>( this );
    objects = findChildren<ConnectionObject*>();
    foreach( ConnectionObject* object, objects )
    { removeAction( &object->connectAction() ); }

    // change ordering
    // this can probably be made faster
    attributesList = dialog.connectionAttributes();
    foreach( const ConnectionAttributes& attributes, attributesList )
    {
        foreach( ConnectionObject* object, objects )
        {
            if( object->connectionAttributes() == attributes )
            { addAction( &object->connectAction() ); }
        }
    }

    _connectionsChanged();
    autoConnect();

}

//____________________________________________________________
void MainMenu::_addConnection( const ConnectionAttributes& attributes )
{

    Debug::Throw( "Network::MainMenu_addConnection.\n" );

    // create new object
    ConnectionObject* object = new ConnectionObject( this );
    object->setConnectionAttributes( attributes );
    connect( object, SIGNAL(messageAvailable(QString)), this, SIGNAL(messageAvailable(QString)) );
    connect( object, SIGNAL(logMessageAvailable(QString)), this, SIGNAL(logMessageAvailable(QString)) );
    connect( object, SIGNAL(connected()), this, SIGNAL(connectionStatusChanged()) );
    connect( object, SIGNAL(disconnected()), this, SIGNAL(connectionStatusChanged()) );

    addAction( &object->connectAction() );

}

//_______________________________________________________
void MainMenu::_installActions( void )
{

    // configuration
    configureAction_ = new QAction( IconEngine::get( IconNames::Configure ), tr( "Configure Connections..." ), this );
    connect( configureAction_, SIGNAL(triggered()), SLOT(_configureConnections()) );

    // reconnect action
    reconnectAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reconnect" ), this ) ;
    reconnectAction_->setShortcut( QKeySequence::Refresh );
    connect( reconnectAction_, SIGNAL(triggered()), SLOT(reconnect()) );

}
