// $Id: MessageBox.cpp,v 1.39 2014/02/10 10:34:30 hupereir Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MessageBox.h"
#include "MessageBox.moc"

#include "CompositeEngine.h"
#include "IconNames.h"
#include "IconSize.h"
#include "IconEngine.h"
#include "Singleton.h"
#include "SvgEngine.h"
#include "XmlOptions.h"
#include "X11Util.h"

#include <QFrame>
#include <QPainter>
#include <QToolButton>
#include <QFrame>

//____________________________________________________________
MessageBox::MessageBox( QWidget* parent, const QString& title ):
    Transparency::TransparentWidget ( parent, Qt::FramelessWindowHint ),
    useSvg_( true ),
    roundCorners_( true )
{

    Debug::Throw( "MessageBox::MessageBox.\n" );
    setWindowFlags( windowFlags() | Qt::WindowStaysOnTopHint );
    setAttribute( Qt::WA_DeleteOnClose, true );
    setMinimumSize( QSize( 250, 0 ) );
    setWindowTitle( tr( "Information - Qssh" ) );

    QVBoxLayout* vlayout = new QVBoxLayout();
    vlayout->setMargin( 0 );
    setLayout( vlayout );

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin( 0 );
    hLayout->setSpacing( 5 );
    vlayout->addLayout( hLayout );

    // title
    hLayout->addWidget( titleLabel_ = new Transparency::ShadowLabel( title, this ) );

    // close button
    QToolButton* button = new QToolButton( this );
    button->setIcon( IconEngine::get( IconNames::DialogClose ) );
    button->setAutoRaise( true );
    connect( button, SIGNAL(pressed()), SLOT(hide()) );
    hLayout->addWidget( button );

    // message frame
    vlayout->addLayout( mainLayout_ = new QVBoxLayout() );

    // configuration
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

    // background
    connect( &Svg::SvgEngine::get(), SIGNAL(changed()), SLOT(_updateMargins()) );
    connect( &Svg::SvgEngine::get(), SIGNAL(changed()), SLOT(setBackgroundChanged()) );

}

//___________________________________________________
void MessageBox::hideFromTaskBar( void )
{

    // hide from taskbar
    #if defined(Q_WS_X11) || defined(Q5_WS_X11)
    X11Util::get().changeProperty( this, X11Util::_NET_WM_STATE_SKIP_TASKBAR, true );
    X11Util::get().changeProperty( this, X11Util::_NET_WM_STATE_SKIP_PAGER, true );
    #endif

}

//___________________________________________________
void MessageBox::addMessage( const QString& message )
{

    int delay( XmlOptions::get().get<int>( "MESSAGE_BOX_DELAY" ) );
    if( delay > 0 )
    {
        Data data( this, message );
        data.timer_->start( 1000*delay, this );
        mainLayout_->addWidget( data.messageLabel_ );
        messages_ << data;
    }

}

//___________________________________________________
void MessageBox::timerEvent( QTimerEvent* event )
{

    Debug::Throw( "MessageBox::timerEvent\n" );
    if( _removeMessage( event->timerId() ) )
    {

        adjustSize();
        if( messages_.empty() ) hide();
        else update();

    } else return Transparency::TransparentWidget::timerEvent( event );

}

//___________________________________________________
void MessageBox::resizeEvent( QResizeEvent *event )
{
    Debug::Throw( "TransparentWindow::resizeEvent.\n" );

    // round corners
    if( roundCorners_ ) setMask( RoundedRegion( rect() ) );

    // update blur region
    if( Transparency::CompositeEngine::get().isAvailable() )
    {
        const QRect rect( _outerPadding().adjustedRect( this->rect() ) );
        _updateBlurRegion( rect.translated( mapTo( window(), QPoint(0,0) ) ) );
    }

    TransparentWidget::resizeEvent( event );
}

//__________________________________________________
bool MessageBox::_removeMessage( int id )
{
    for( MessageList::iterator iter = messages_.begin(); iter != messages_.end(); ++iter )
    {
        if( iter->timer_->timerId() != id ) continue;
        iter->timer_->stop();
        iter->messageLabel_->hide();
        iter->messageLabel_->deleteLater();
        messages_.erase( iter );
        return true;
    }

    return false;

}

//__________________________________________________
void MessageBox::_paint( QPaintDevice& device, const QRect& rect )
{

    QPainter painter( &device );
    painter.setClipRect( rect );
    painter.setPen( palette().color( QPalette::Text ) );

    if( Transparency::CompositeEngine::get().isAvailable() )
    {
        painter.setRenderHints(QPainter::SmoothPixmapTransform);
        painter.setCompositionMode(QPainter::CompositionMode_Source );
    }

    if( useSvg_ && Svg::SvgEngine::get().isValid() )
    { painter.drawPixmap( QPoint(0,0), Svg::SvgEngine::get().get( size() ) ); }

    painter.end();

}


//________________________________________________
void MessageBox::_render( QPaintDevice& device, QWidget& widget, const QRect& clip )
{ widget.render( &device, widget.geometry().topLeft(), widget.rect().intersected( clip.translated( -widget.geometry().topLeft() ) ), 0 ); }

//________________________________________________
void MessageBox::_updateConfiguration( void )
{
    Debug::Throw( "MessageBox::_updateConfiguration.\n" );

    useSvg_ = XmlOptions::get().get<bool>( "USE_SVG" );
    roundCorners_ = XmlOptions::get().get<bool>( "ROUND_CORNERS" );

    QPalette palette( MessageBox::palette() );
    palette.setColor( QPalette::Text, foregroundColor() );
    palette.setColor( QPalette::WindowText, foregroundColor() );
    setPalette( palette );

    titleLabel_->setShadowOffset( shadowOffset() );
    QColor shadowColor( this->shadowColor() );
    if( shadowColor.isValid() ) titleLabel_->setShadowColor( shadowColor );

    // svg
    if( useSvg_ && Svg::SvgEngine::get().reload( true ) )
    { setBackgroundChanged( true ); }

    // margins
    _updateMargins();

    // set font to bold
    QFont bold( titleLabel_->font() );
    bold.setWeight( QFont::Bold );
    titleLabel_->setFont( bold );

}

//________________________________________________
void MessageBox::_updateMargins( void )
{
    Debug::Throw( "MessageBox::_updateMargins.\n" );

    // margins
    _clearMargins();
    Base::Margins margins;
    if( useSvg_ && Svg::SvgEngine::get().isValid() )
    { margins = Svg::SvgEngine::get().margins(); }

    margins += Base::Margins( XmlOptions::get().get<int>( "WINDOW_BORDER" ) );
    setContentsMargins( margins.left(), margins.top(), margins.right(), margins.bottom() );
    _setMargins( margins );

    // outer padding
    _clearOuterPadding();
    if( useSvg_ && Svg::SvgEngine::get().isValid() )
    {

        // read either outer padding or margins from SVG, and adjust
        Base::Margins outerPadding( Svg::SvgEngine::get().outerPadding() );
        if( outerPadding.isNull() )
        {
            outerPadding = Svg::SvgEngine::get().margins();
            if( !outerPadding.isNull() ) outerPadding -= Base::Margins( 4 );
        }

        _setOuterPadding( outerPadding );

    }

}


//____________________________________
MessageBox::Data::Data( QWidget* parent, const QString& message ):
    Counter( "MessageBox::Data" ),
    timer_( new QBasicTimer() )
{

    Transparency::TransparentWidget *widget( qobject_cast<Transparency::TransparentWidget*>( parent ) );
    Q_CHECK_PTR( widget );

    Transparency::ShadowLabel* label = new Transparency::ShadowLabel( parent );
    label->setShadowOffset( widget->shadowOffset() );
    label->setText( message );

    QColor shadowColor( widget->shadowColor() );
    if( shadowColor.isValid() ) label->setShadowColor( shadowColor );

    messageLabel_ = label;
}
