// $Id: SystemTrayIcon.cpp,v 1.29 2014/02/03 10:08:27 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "SystemTrayIcon.h"
#include "SystemTrayIcon.moc"

#include "Application.h"
#include "ConnectionObject.h"
#include "Debug.h"
#include "IconSize.h"
#include "MainMenu.h"
#include "Menu.h"
#include "Singleton.h"
#include "SystemNotifications.h"
#include "ToolTipWidget.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>

//______________________________________________________
SystemTrayIcon::SystemTrayIcon( QObject* parent ):
    QSystemTrayIcon( parent ),
    Counter( "SystemTrayIcon" ),
    systemNotifications_( 0 ),
    toolTipWidget_( 0 )
{

    Debug::Throw( "SystemTrayIcon::SystemTrayIcon.\n" );

    // use application icon
    setIcon( static_cast<const Application*>( Singleton::get().application() )->applicationIcon() );

    // main menu
    mainMenu_ = new MainMenu();
    connect( mainMenu_, SIGNAL(connectionStatusChanged()), SLOT(_updateToolTip()) );
    mainMenu_->readConnections();

    // context menu
    setContextMenu( menu_ = new Menu() );
    menu_->insertAction( &menu_->separator(), &mainMenu_->configurationAction() );

    // tooltip widget
    toolTipWidget_ = new ToolTipWidget( 0 );
    toolTipWidget_->setPreferredPosition( ToolTipWidget::Top );

    // connection
    // forward message available signal
    connect( mainMenu_, SIGNAL(messageAvailable(QString)), this, SLOT(_notify(QString)) );
    connect( this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(_activate(QSystemTrayIcon::ActivationReason)) );

}

//______________________________________________________
SystemTrayIcon::~SystemTrayIcon( void )
{
    Debug::Throw( "SystemTrayIcon::~SystemTrayIcon.\n" );
    mainMenu_->deleteLater();
    menu_->deleteLater();
    toolTipWidget_->deleteLater();
}

//______________________________________________________
bool SystemTrayIcon::supportsSystemNotifications( void )
{ return supportsMessages() || SystemNotifications::isSupported(); }

//_______________________________________________
bool SystemTrayIcon::event(QEvent *event)
{

    if (event->type() == QEvent::ToolTip )
    {
        const int offset( 10 );

        _updateToolTip();
        toolTipWidget_->setIndexRect( geometry().adjusted( -offset, -offset, offset, offset ) );
        toolTipWidget_->adjustSize();
        toolTipWidget_->show();
    }

    return QSystemTrayIcon::event(event);

}

//______________________________________________________
void SystemTrayIcon::_activate( QSystemTrayIcon::ActivationReason reason )
{

    Debug::Throw( "SystemTrayIcon::_activate.\n" );
    if( reason == Trigger )
    {

        if( mainMenu_->isVisible() ) mainMenu_->hide();
        else {

            // calculate position based on icon geometry
            QSize size( mainMenu_->sizeHint() );
            QRect geometry( SystemTrayIcon::geometry() );

            // retrieve desktop
            QDesktopWidget *desktop( qApp->desktop() );
            QSize desktop_size( desktop->frameSize() );

            static const int margin = 5;
            int x(0);
            int y(0);

            // vertical position
            if( ( geometry.top() - size.height() - margin ) >= 0 ) y = geometry.top() - size.height() - margin;
            else if( geometry.bottom() + size.height() + margin <= desktop_size.height() ) y = geometry.bottom() + margin;
            else y = desktop_size.height() - size.height();

            // horizontal position
            if( geometry.left() + size.width() + margin <= desktop_size.width() ) x = geometry.left();
            else x = desktop_size.width() - size.width() - margin;

            mainMenu_->setGeometry( QRect( QPoint( x, y ), size ) );
            mainMenu_->show();
        }

    } else return;

}

//__________________________________________________________________
void SystemTrayIcon::_updateToolTip( void )
{
    Debug::Throw( "SystemTrayIcon::_updateToolTip.\n" );

    toolTipWidget_->setPixmap(
        static_cast<const Application*>( Singleton::get().application() )->applicationIcon().pixmap( IconSize( IconSize::Maximum ) ) );

    // html formated tooltip
    // title
    QString buffer;
    QTextStream what( &buffer );
    what << "<div align='center' style='border: 10px;'><b>Qssh</b></div>" << endl;
    what << "<table style='margin: 10px;' >" << endl;

    QList<ConnectionObject*> connections( mainMenu_->findChildren<ConnectionObject*>() );
    if( connections.empty() )
    {

        what << tr( "<tr><td align='center'>No connections configured</tr></tr>" ) << endl;

    } else {

        foreach( ConnectionObject* connection, connections )
        {
            what
                << "<tr><td align='right'>" << connection->connectionAttributes().name() << ": </td><td>"
                << (connection->connectAction().isChecked() ? tr( "connected" ): tr( "disconnected") ) << "</td></tr>" << endl;

        }

    }

    what << "</table>" << endl;

    toolTipWidget_->setText( buffer );

    return;

}

//______________________________________________________
void SystemTrayIcon::_notify( QString message )
{
    if( supportsSystemNotifications() && XmlOptions::get().get<bool>( "USE_SYSTEM_NOTIFICATIONS" ) )
    {

        if( SystemNotifications::isSupported() )
        {

            if( !systemNotifications_ )
            {

                const Application& application( *static_cast<const Application*>( Singleton::get().application() ) );
                systemNotifications_ = new SystemNotifications( this, application.applicationName(), application.applicationIcon() );
            }

            systemNotifications_->processMessage( tr( "Ssh connection state changed" ), message );

        } else {

            showMessage( tr( "Ssh connection status" ), message );

        }

    } else emit messageAvailable( message );
}
